#pragma once

#include <vector>
#include <cmath>
using namespace std;

/** Bezier characteristic matrix */
extern double BZ[4][4];
/* Inverse of Bezier characteristic matrix*/
extern double iBZ[4][4];
/** Catmull-Rom characteristic matrix.  */
extern double CR[4][4];

class CCurve
{
public:

	CCurve(void) ;

	CCurve(CCurve& copy);

	~CCurve(void);

	void AddPoint(const POINT point) {
		m_points.push_back(point);
	}
	int GetNumPoints() const {
		return (int)m_points.size();
	}
	void Clear() {
		m_points.clear();
	}
	void Finalize() {
		m_finalized = true;
	}
	bool IsFinalized() const {
		return m_finalized;
	}

	void Draw(CDC& dc);
	void DrawDragImage(CDC& dc, CPoint pt);

	bool IsHit(const CPoint& point);
	
	const vector<POINT>& GetPoints() const { return m_points;}

	CRect GetBoundingBox() const 
	{
		int minX=9999999, minY=99999999, maxX=-999999999, maxY=-99999999;
		unsigned int numPoints = m_points.size(); 
		for (unsigned int i=0;i<numPoints;i++)
		{
			if (m_points[i].x < minX)
				minX = m_points[i].x;
			if (m_points[i].x > maxX)
				maxX = m_points[i].x;

			if (m_points[i].y < minY)
				minY = m_points[i].y;
			if (m_points[i].y > maxY)
				maxY = m_points[i].y;
		}
		return CRect(minX,minY,maxX,maxY);
		
	}

	void MoveCurve(CPoint& diff);
	void MovePoint(int index, CPoint& diff);

	void NormalizeToBoundingBox();

	
private:
	bool m_finalized;
	vector<POINT> m_points;

	vector<POINT> GetBezierPoints() ;
	
	int Distance(const POINT p1, const POINT p2) {
		int dx = p2.x - p1.x;
		int dy = p2.y - p1.y;
		return (int)sqrt((double)(dy*dy+dx*dx));
	}

	void Multiply(double a[4][4], double b[4][4], double result[4][4]);

	void Multiply(double matrix[4][4],POINT point, POINT& result);

	void Multiply(double matrix[4][4], double vals[4], double results[4]);
};
