// Curvy2View.cpp : implementation of the CCurvy2View class
//

#include "stdafx.h"
#include "Curvy2.h"

#include "Curvy2Doc.h"
#include "Curvy2View.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CCurvy2View

IMPLEMENT_DYNCREATE(CCurvy2View, CView)

BEGIN_MESSAGE_MAP(CCurvy2View, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, &CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CView::OnFilePrintPreview)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_CURVES_DRAW, &CCurvy2View::OnCurvesDraw)
	ON_UPDATE_COMMAND_UI(ID_CURVES_DRAW, &CCurvy2View::OnUpdateCurvesDraw)
	ON_COMMAND(ID_CURVES_SELECT, &CCurvy2View::OnCurvesSelect)
	ON_UPDATE_COMMAND_UI(ID_CURVES_SELECT, &CCurvy2View::OnUpdateCurvesSelect)
	ON_WM_MOUSEMOVE()
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, &CCurvy2View::OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, &CCurvy2View::OnUpdateEditCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, &CCurvy2View::OnUpdateEditPaste)
	ON_WM_CREATE()
END_MESSAGE_MAP()

// CCurvy2View construction/destruction

CCurvy2View::CCurvy2View()
: m_bDrawing(false)
, m_bReadyToDrag(false)
, m_bDraggingCurve(false)
, m_bDraggingPoint(false)
, m_ptOldPoint(0)
, m_ptLastMsg(0)
, m_ptLastImage(0)
, m_ptOffset(0)
, m_tempCurve(NULL)
, m_cmCommandMode(Draw)
{
	// TODO: add construction code here

}

CCurvy2View::~CCurvy2View()
{
}

BOOL CCurvy2View::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

// CCurvy2View drawing

void CCurvy2View::OnDraw(CDC* pDC)
{
	CCurvy2Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	if (!pDoc)
		return;

	DrawCurves(pDC, pDoc);

}


// CCurvy2View printing

BOOL CCurvy2View::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CCurvy2View::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CCurvy2View::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CCurvy2View diagnostics

#ifdef _DEBUG
void CCurvy2View::AssertValid() const
{
	CView::AssertValid();
}

void CCurvy2View::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CCurvy2Doc* CCurvy2View::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCurvy2Doc)));
	return (CCurvy2Doc*)m_pDocument;
}
#endif //_DEBUG


// CCurvy2View message handlers

void CCurvy2View::DrawCurves(CDC* pDC, CCurvy2Doc* pDoc)
{
	
	unsigned int nNumCurves = pDoc->GetNumCurves();
	for (unsigned int i=0;i < nNumCurves ; i++)
	{
		CCurve* curve = pDoc->GetCurve(i);
		curve->Draw(*pDC);
		
		//draw selection boxes for selected curve
		if (!m_bDrawing && pDoc->GetSelectedCurve() != NULL)
		{
			
			CBrush brush(RGB(0,0,0));
			vector<POINT> points = pDoc->GetSelectedCurve()->GetPoints();
			int numPoints = points.size();
			//draw selection boxes
			for (int i=0;i<numPoints;i++)
			{
				CRect rect(points[i],points[i]);
				rect.InflateRect(3,3);

				pDC->FillRect(rect,&brush);
			}

			//draw center drag icon
			CRect rect = pDoc->GetSelectedCurve()->GetBoundingBox();
			CRect ctr(rect.CenterPoint(),rect.CenterPoint());
			ctr.InflateRect(4,4);
			pDC->FillRect(ctr,&brush);
			brush.DeleteObject();
						
		}
	}
}

void CCurvy2View::OnLButtonDown(UINT nFlags, CPoint point)
{
	CCurvy2Doc* pDoc = GetDocument();
	
	CCurve* curve = NULL;
	CCurve* hitCurve = pDoc->GetClosestCurve(point);
	
	switch (m_cmCommandMode)
	{
	case Select:
		//test against control points
		if (pDoc->GetSelectedCurve() != NULL)
		{
			bool bSelected = pDoc->SelectControlPoint(pDoc->GetSelectedCurve(), point);
			if (bSelected)
			{
				m_ptOldPoint = point;
				m_bDraggingPoint = true;
				
			} 

		}
		//test for selecting a different curve
		if (hitCurve != pDoc->GetSelectedCurve())
		{
			pDoc->SelectCurve( hitCurve );
			pDoc->UpdateAllViews(NULL);
			
		}

		//test for moving the selected curve
		if (pDoc->GetSelectedCurve()) {
			
			CRect rect = pDoc->GetSelectedCurve()->GetBoundingBox();
			if (rect.PtInRect(point))
			{
				m_ptOldPoint = point;
				m_bReadyToDrag = true;
			}
		}
		break;
	case Draw:
		if (pDoc->GetSelectedCurve() == NULL || pDoc->GetSelectedCurve()->IsFinalized())
		{
			pDoc->SelectCurve(pDoc->AddNewCurve()) ;
			m_bDrawing = true;
		}

		pDoc->GetSelectedCurve()->AddPoint(point);
		pDoc->UpdateAllViews(NULL);
		
		break;
	default:
		ASSERT(0);
	}

	CView::OnLButtonDown(nFlags, point);
}

void CCurvy2View::OnRButtonDown(UINT nFlags, CPoint point)
{
	CCurvy2Doc* pDoc = GetDocument();

	if (m_bDrawing && pDoc->GetSelectedCurve() != NULL)
	{
		pDoc->GetSelectedCurve()->Finalize();
		pDoc->UpdateAllViews(NULL);
	}
	m_bDrawing = false;

	CView::OnRButtonDown(nFlags, point);
}

void CCurvy2View::OnLButtonUp(UINT nFlags, CPoint point)
{
	
	m_bDraggingPoint = false;
	
	m_bReadyToDrag = false;

	CView::OnLButtonUp(nFlags, point);
}

void CCurvy2View::OnMouseMove(UINT nFlags, CPoint point)
{
	CCurvy2Doc* pDoc = GetDocument();
	if (m_bDraggingPoint)
	{
		CPoint diff = point - m_ptOldPoint;
		pDoc->MoveSelectedControlPoint(diff);
		m_ptOldPoint = point;
		Invalidate();
	}

	else if (m_bReadyToDrag)
	{
		m_bReadyToDrag = false;
		m_bDraggingCurve = true;
		HANDLE hData = pDoc->CreateClipboardFormatCurvy(pDoc->GetSelectedCurve(), false);
		COleDataSource ods;
		ods.CacheGlobalData(pDoc->GetClipFormat(), hData);

		CCurve* pOldCurve = pDoc->GetSelectedCurve();

		DROPEFFECT de = ods.DoDragDrop(DROPEFFECT_COPY | DROPEFFECT_MOVE);

		m_bDraggingCurve = false;

		if (de == DROPEFFECT_MOVE)
		{
			pDoc->RemoveCurve(pOldCurve);
		}
		
	}

	CView::OnMouseMove(nFlags, point);
}

void CCurvy2View::OnCurvesDraw()
{
	m_cmCommandMode = Draw;
}

void CCurvy2View::OnUpdateCurvesDraw(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(m_bDraggingPoint==false && m_bDraggingCurve==false);
	pCmdUI->SetRadio(m_cmCommandMode == Draw);
}

void CCurvy2View::OnCurvesSelect()
{
	m_cmCommandMode = Select;
}

void CCurvy2View::OnUpdateCurvesSelect(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(TRUE);
	pCmdUI->SetRadio(m_cmCommandMode == Select);
}



//BOOL CCurvy2View::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
//{
//	m_oleDropTarget.Register(this);
//
//	return CView::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
//}

DROPEFFECT CCurvy2View::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	CView::OnDragEnter(pDataObject, dwKeyState, point);
	CCurvy2Doc* pDoc = GetDocument();

	if (!pDataObject->IsDataAvailable(pDoc->GetClipFormat()))
		return DROPEFFECT_NONE;

	HANDLE hData = pDataObject->GetGlobalData(pDoc->GetClipFormat());

	if (hData)
	{
		m_tempCurve = pDoc->CreateCurveFromHandle(hData);
		CRect rect = m_tempCurve->GetBoundingBox();

		int x = point.x - rect.Width()/2;
		int y = point.y - rect.Height()/2;

		m_ptLastMsg = point;
		m_ptLastImage.x = m_ptLastImage.y = -32000;
		if (rect.PtInRect(point)) {
			m_ptOffset.x = (point.x - rect.left);
			m_ptOffset.y = (point.y - rect.top) ;
		}
		else
		{
			//shape isn't under arrow--move
			m_ptOffset.x = rect.Width()/2;
			m_ptOffset.y = rect.Height()/2;
		}
		m_tempCurve->NormalizeToBoundingBox();

		return (dwKeyState & MK_CONTROL) ? DROPEFFECT_COPY : DROPEFFECT_MOVE;
	}
	m_tempCurve = NULL;
	return DROPEFFECT_NONE;
}

void CCurvy2View::OnDragLeave()
{
	CView::OnDragLeave();

	if (m_tempCurve != NULL)
	{
		CClientDC dc(this);
		OnPrepareDC(&dc);
		m_tempCurve->DrawDragImage(dc,m_ptLastImage);
		delete m_tempCurve;
		m_tempCurve = NULL;
		Invalidate();
	}
}

DROPEFFECT CCurvy2View::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	CView::OnDragOver(pDataObject,dwKeyState,point);
	CCurvy2Doc* pDoc = GetDocument();
	if (!pDataObject->IsDataAvailable(pDoc->GetClipFormat()))
		return DROPEFFECT_NONE;

	if (m_tempCurve==NULL)
		return DROPEFFECT_NONE;

	CClientDC dc(this);
	OnPrepareDC(&dc);
	
	if (point != m_ptLastMsg)
	{
		CPoint pt ( point.x - m_ptOffset.x, point.y - m_ptOffset.y);
				
		m_tempCurve->DrawDragImage(dc, m_ptLastImage);
		m_tempCurve->DrawDragImage(dc, pt);
		

		m_ptLastImage = pt;
		m_ptLastMsg = point;
	}

	return (dwKeyState & MK_CONTROL) ? DROPEFFECT_COPY : DROPEFFECT_MOVE;
}

BOOL CCurvy2View::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	CView::OnDrop(pDataObject,dropEffect,point);
	CCurvy2Doc* pDoc = GetDocument();

	CClientDC dc(this);
	OnPrepareDC(&dc);
	
	if (m_tempCurve!=NULL)
	{
		m_tempCurve->DrawDragImage(dc,m_ptLastImage);
		delete m_tempCurve;
		m_tempCurve = NULL;
	}
	
	HANDLE hData = pDataObject->GetGlobalData(pDoc->GetClipFormat());
	if (hData!=NULL)
	{
		CCurve* newCurve = pDoc->CreateCurveFromHandle(hData);
		if (newCurve) {
			CPoint pt(point.x-m_ptOffset.x, point.y - m_ptOffset.y);
			newCurve->NormalizeToBoundingBox();
			newCurve->MoveCurve(pt);
			pDoc->AddCurve(newCurve);
			pDoc->SelectCurve(newCurve);
			pDoc->UpdateAllViews(NULL);
			m_cmCommandMode = Select;
		}
		
		::GlobalFree(hData);
		return TRUE;
	} 
	return FALSE;
}

void CCurvy2View::OnUpdateEditCopy(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(m_bDrawing == false && GetDocument()->GetSelectedCurve() != NULL);
}

void CCurvy2View::OnUpdateEditCut(CCmdUI *pCmdUI)
{
	pCmdUI->Enable(m_bDrawing == false && GetDocument()->GetSelectedCurve() != NULL);
}

void CCurvy2View::OnUpdateEditPaste(CCmdUI *pCmdUI)
{
	COleDataObject odo;
	odo.AttachClipboard();
	pCmdUI->Enable(odo.IsDataAvailable(GetDocument()->GetClipFormat()) && m_bDrawing == false);
}

int CCurvy2View::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	this->m_oleDropTarget.Register(this);

	return 0;
}
