// Curvy2Doc.cpp : implementation of the CCurvy2Doc class
//

#include "stdafx.h"
#include "Curvy2.h"

#include "Curvy2Doc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CCurvy2Doc

IMPLEMENT_DYNCREATE(CCurvy2Doc, CDocument)

BEGIN_MESSAGE_MAP(CCurvy2Doc, CDocument)
	ON_COMMAND(ID_EDIT_COPY, &CCurvy2Doc::OnEditCopy)
	ON_COMMAND(ID_EDIT_CUT, &CCurvy2Doc::OnEditCut)
	ON_COMMAND(ID_EDIT_PASTE, &CCurvy2Doc::OnEditPaste)
END_MESSAGE_MAP()


// CCurvy2Doc construction/destruction

CCurvy2Doc::CCurvy2Doc()
: m_selectedCurve(NULL)
, m_nCurrentControlPoint(-1)
{
	m_clipboardFormat = ::RegisterClipboardFormat(_T("Ben.Curvy"));
}

CCurvy2Doc::~CCurvy2Doc()
{
	DeleteContents();
}

BOOL CCurvy2Doc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

void CCurvy2Doc::DeleteContents()
{
	int numCurves = m_curves.size();
	for (int curve = 0; curve < numCurves;curve++)
	{
		delete m_curves[curve];
	}
	m_curves.clear();
	m_selectedCurve = NULL;
	UpdateAllViews(NULL);

}


// CCurvy2Doc serialization

void CCurvy2Doc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		unsigned int nNumCurves = m_curves.size();
		ar << nNumCurves ;
		for (unsigned int i =0; i<nNumCurves ; i++)
		{
			CCurve* curve = m_curves[i];
			unsigned int nNumPoints = curve->GetNumPoints();

			ar << nNumPoints;

			const vector<POINT>& points = curve->GetPoints();

			for (unsigned int pt = 0; pt < nNumPoints; pt++)
			{
				POINT point = points[pt];
				ar << point.x << point.y;
			}
		}

	}
	else
	{
		unsigned int nNumCurves = 0;
		ar >> nNumCurves;
		for (unsigned int i=0;i<nNumCurves;i++)
		{
			unsigned int nNumPoints = 0;
			ar >> nNumPoints;

			CCurve* curve = new CCurve();

			for (unsigned int pt = 0; pt < nNumPoints; pt++)
			{
				
				POINT point;
				ar >> point.x >> point.y;
				curve->AddPoint(point);
			}
			AddCurve(curve);
		}
	}
}


// CCurvy2Doc diagnostics

#ifdef _DEBUG
void CCurvy2Doc::AssertValid() const
{
	CDocument::AssertValid();
}

void CCurvy2Doc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


// CCurvy2Doc commands

void CCurvy2Doc::AddCurve(CCurve* newCurve)
{
	m_curves.push_back(newCurve);
	
}

void CCurvy2Doc::RemoveCurve(CCurve* curve)
{
	if (curve==NULL)
		return;

	vector<CCurve*>::iterator p = m_curves.begin();
	while (p!=m_curves.end()) {
		CCurve* c = *p;
		if (c == curve) {
			m_curves.erase(p);
			delete curve;
			UpdateAllViews(NULL);
			break;
		}
		p++;
	}
}



CCurve* CCurvy2Doc::GetClosestCurve(CPoint& point)
{
	int numCurves = m_curves.size();
	for (int i=numCurves-1;i>=0;i--)
	{
		CCurve* c = m_curves[i];
		if (c->IsHit(point))
		{
			return c;
		}
	}
	return NULL;
}

bool CCurvy2Doc::SelectControlPoint(CCurve* curve, CPoint& point)
{
	//check for move on each point's adjust widget
	const vector<POINT>& points = curve->GetPoints();
	unsigned int numPoints = points.size();
	for (unsigned int i=0;i<numPoints;i++)
	{
		CRect rect(points[i],points[i]);
		rect.InflateRect(4,4);

		if (rect.PtInRect(point))
		{
			this->m_nCurrentControlPoint = static_cast<int>(i);
			return true;
		}
	}
	m_nCurrentControlPoint = -1;
	return false;
}

void CCurvy2Doc::OnEditCopy()
{
	ASSERT(m_selectedCurve != NULL);
	COleDataSource* pods = new COleDataSource;
	//create native format for this program
	HANDLE hData = CreateClipboardFormatCurvy(m_selectedCurve,false);
	if (hData) {
		pods->CacheGlobalData(m_clipboardFormat, hData);
	}		

	//create bitmap to insert into other programs
	hData = CreateClipboardFormatBitmap(m_selectedCurve);
	if (hData) {
		pods->CacheGlobalData(CF_DIB, hData);
	}
	pods->SetClipboard();
}

void CCurvy2Doc::OnEditCut()
{
	OnEditCopy();

	RemoveCurve(m_selectedCurve);
	m_selectedCurve = NULL;
	
	UpdateAllViews(NULL);
}

void CCurvy2Doc::OnEditPaste()
{
	COleDataObject odo;
	odo.AttachClipboard();
	HANDLE hData = odo.GetGlobalData(m_clipboardFormat);

	if (hData != NULL)
	{
		CCurve* newCurve = CreateCurveFromHandle(hData);
		if (newCurve) {
			m_curves.push_back(newCurve);
			SelectCurve(newCurve);
		}
			
		GlobalFree(hData);
		
		UpdateAllViews(NULL);
	}
	
}

HANDLE CCurvy2Doc::CreateClipboardFormatBitmap(CCurve* curve, bool bNormalize)
{
	//create a curve normalized to it's bounding box
	CCurve newCurve(*curve);
	if (bNormalize)
		newCurve.NormalizeToBoundingBox();
	CRect boundingBox = newCurve.GetBoundingBox();
	boundingBox.InflateRect(5,5);

	//draw curve on to a bitmap

	CClientDC dcScreen(this->GetRoutingView());
	CBitmap bitmap;
	bitmap.CreateCompatibleBitmap(&dcScreen,boundingBox.Width(),boundingBox.Height());
    
	CDC dcMem;
	dcMem.CreateCompatibleDC(&dcScreen);
	CBitmap* pOldBitmap = dcMem.SelectObject(&bitmap);
	dcMem.FillSolidRect(0,0,boundingBox.Width(),boundingBox.Height(),RGB(255,255,255));
	newCurve.Draw(dcMem);
	dcMem.SelectObject(pOldBitmap);

	int Bpp = 3;
	BITMAPINFO pRes;
	::ZeroMemory(&pRes, sizeof(BITMAPINFO));
	pRes.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	pRes.bmiHeader.biWidth = boundingBox.Width();
	pRes.bmiHeader.biHeight = boundingBox.Height();
	pRes.bmiHeader.biPlanes = 1;
	pRes.bmiHeader.biBitCount = Bpp * 8;
	pRes.bmiHeader.biSizeImage = ((Bpp * boundingBox.Width() + Bpp) & ~3)
		* boundingBox.Height();

	HANDLE hData = ::GlobalAlloc(GMEM_MOVEABLE, pRes.bmiHeader.biSizeImage + pRes.bmiHeader.biSize);
	BYTE* pData = (BYTE*)::GlobalLock(hData);
	memcpy(pData,&pRes,pRes.bmiHeader.biSize);
	::GetDIBits(dcMem.GetSafeHdc(),
		(HBITMAP)bitmap,0,boundingBox.Height(),pData + pRes.bmiHeader.biSize,&pRes,DIB_RGB_COLORS);

	::GlobalUnlock(hData);
	return hData;
}

HANDLE CCurvy2Doc::CreateClipboardFormatCurvy(CCurve* curve, bool bNormalize)
{

	CCurve newCurve(*curve);
	if (bNormalize)
		newCurve.NormalizeToBoundingBox();
	
	int numPoints = newCurve.GetNumPoints();
	int size = (numPoints * sizeof(POINT)) + 4;

	HANDLE hData = ::GlobalAlloc(GMEM_MOVEABLE, size);
	BYTE* pData = (BYTE*)::GlobalLock(hData);
	
	memcpy(pData,&size,4);
	int offset = 4;
	vector<POINT> pts = newCurve.GetPoints();
	for (int i=0;i<numPoints;i++) {
		POINT pt = pts[i];
        memcpy(pData+offset,&pt,sizeof(pt));
		offset+=sizeof(pt);
	}

	::GlobalUnlock(hData);
	return hData;
}

CCurve* CCurvy2Doc::CreateCurveFromHandle(HANDLE hData)
{
	if (hData==NULL)
		return NULL;

	BYTE* pData = (BYTE*)::GlobalLock(hData);
	if (pData!=NULL)
	{
		CCurve* newCurve = new CCurve();
		int size;
		memcpy(&size,pData,4);
		int numPoints = (size - 4)/sizeof(POINT);
		int offset = 4;
		for (int i=0;i<numPoints;i++)
		{
			POINT pt;
			memcpy(&pt,pData+offset,sizeof(POINT));
			newCurve->AddPoint(pt);
			offset+=sizeof(POINT);
		}
		newCurve->Finalize();

		::GlobalUnlock(hData);
		
		return newCurve;
	}
	return NULL;
}

CCurve* CCurvy2Doc::SelectCurve(CPoint& point)
{
	m_selectedCurve = this->GetClosestCurve(point);
	
	return m_selectedCurve;
}

void CCurvy2Doc::MoveSelectedControlPoint(CPoint& diff)
{
	if (m_selectedCurve != NULL)
	{
		m_selectedCurve->MovePoint(this->m_nCurrentControlPoint, diff);
		UpdateAllViews(NULL);
	}
}
